local addonInfo, privateVars = ...

if nkCurrency ~= nil then return end
nkCurrency = {}

privateVars.uiElements = {}
privateVars.uiElements.context = UI.CreateContext("nkCurrency")
privateVars.uiElements.context:SetStrata ('dialog')

function nkCurrency.main(_, addon)

	if addon == addonInfo.identifier then
		
		table.insert(Command.Slash.Register("nkCur"), {nkCurrency.commandHandler, "nkCurrency", "config"})
		
		privateVars.currency = {}
		
		if nkPanel ~= nil then
			privateVars.uiElements.ui = nkCurrency.nkPanelPlugin()
			nkPanel.api.registerPlugin('nkCurrency', privateVars.uiElements.ui)
		else		
			privateVars.uiElements.ui = nkCurrency.ui()
			local items = {	{ label = "Show window", callBack = function () nkCTSetup.visible = true; privateVars.uiElements.ui:SetVisible(true) end} }			
			EnKai.manager.init('nkCurrency', items, nil)
		end
				
		Command.Event.Attach(Event.Currency, nkCurrency.CurrencyChange, "nkCurrency.Currency")
		
		EnKai.version.init(addonInfo.toc.Identifier, addonInfo.toc.Version)
	end
	
end

function nkCurrency.settingsHandler(_, addon) 

	if addon == addonInfo.identifier then
	
		if nkCTSetup == nil then
			nkCTSetup = { xpos = 800, ypos = 200, autoclose = true, dragable = true, visible = true}
		end		
	end
	
	-- Version 1.0.2 setup changes
	
	if nkCTSetup.autoclose == nil then nkCTSetup.autoclose = true end
	if nkCTSetup.dragable == nil then nkCTSetup.dragable = true end
	
	-- Version 1.0.3 configuration change
		
	if nkCTSetup.visible == nil then nkCTSetup.visible = true end  
	
	-- Version 1.1.3 configuration change
	
	if nkCTSetup.titleCurrency == nil then nkCTSetup.titleCurrency = 'coin' end
					
end

function nkCurrency.commandHandler(commandline)

	if privateVars.uiElements.ui:GetVisible() == true then
		privateVars.uiElements.ui:SetVisible(false)
		nkCTSetup.visible = false
	else
		privateVars.uiElements.ui:SetVisible(true)
		nkCTSetup.visible = true
	end
		
end

Command.Event.Attach(Event.Addon.Load.End, nkCurrency.main, "nkCurrency.Addon.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, nkCurrency.settingsHandler, "nkCurrency.SavedVariables.Load.End")

--------------------------------------------------------------------------------------------------------------------------------------------

function nkCurrency.CurrencyChange (_, currency)

	-- workaround for Inspect.Currency.Category.List() sometimes not delivering all currencies
	
	if #privateVars.uiElements.ui.categorySelect:GetValue('selection') <= 1 then	
		local selection = {}
		for k, v in pairs (Inspect.Currency.Category.List()) do
			local details = Inspect.Currency.Category.Detail(k)
			if details ~= nil then
				table.insert(selection, { label = details.name, value = details.id})
			end
		end
		
		privateVars.uiElements.ui.categorySelect:SetSelection(selection)
	end
	
	local showUI = false
	
	for k, v in pairs (currency) do
		local details = Inspect.Currency.Detail(k)
		if details ~= nil then
			if privateVars.currency[details.category] == nil then privateVars.currency[details.category] = {} end
			
			if details.category == nkCTSetup.selectedCurrency then showUI = true end	
		
			local max = details.stackMax
			if max == nil then max = "" else max = EnKai.strings.formatNumber(max) end
		
			if k == 'coin' then
				local platin = math.floor(v / 10000)
				local gold = math.floor((v - (platin * 10000)) / 100)
				local silver = v - (platin * 10000) - (gold * 100) 
				privateVars.currency[details.category][k] = { name = details.name, value = string.format("%dP, %dG, %dS", platin, gold, silver), max = max }
			else					
				privateVars.currency[details.category][k] = { name = details.name, value = v, max = max }
			end
			
			if nkCTSetup.titleCurrency == k then
				privateVars.uiElements.ui:SetTitle(string.format("%s: %s", privateVars.currency[details.category][k].name, privateVars.currency[details.category][k].value))
				
				if nkPanel == nil then privateVars.uiElements.ui:SetTitleAlign("right", - 25) end
			end
		end
	end
	
	if showUI == true then
	
		local gridValues = {}
		
		local currentCategory = privateVars.uiElements.ui.categorySelect:GetSelectedValue()
		
		for k, v in pairs (privateVars.currency[currentCategory]) do
			local value = v.value
			if k ~= 'coin' then
				value = EnKai.strings.formatNumber (value)
			end
			table.insert (gridValues, {{ key = k, value = v.name }, { value = value }, { value = v.max } })
		end
		
		table.sort(gridValues, function (a, b) return a[1].value < b[1].value end)
		
		privateVars.uiElements.ui:PopulateGrid(gridValues)
		privateVars.uiElements.ui:ShowGrid(true)
	end

end


--------------------------------------------------------------------------------------------------------------------------------------------

function nkCurrency.ui ()

	local name = 'nkCurrency'

	local ui = EnKai.uiCreateFrame("nkWindowElement", name .. ".ui", privateVars.uiElements.context)
	ui:SetWidth(398)
	ui:SetHeight(123)
	ui:SetBackgroundColor(0, 0, 0, 0)
	ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nkCTSetup.xpos, nkCTSetup.ypos)
	ui:SetTitleAlign ("right", -80)
	ui:SetTitle(string.format("nkCurrency V%s", addonInfo.toc.Version))
	ui:SetAutoHide(true, 5)
	ui:SetFontSize(13)
	ui:SetDragable(nkCTSetup.dragable)
	ui:ShowMoveToggle(true)
	ui:ShowAutoHideToggle(true)
	ui:SetVisible(nkCTSetup.visible)
	
	Command.Event.Attach(EnKai.events[name .. '.ui'].Moved, function (_, xpos, ypos)
		nkCTSetup.xpos = xpos
	    nkCTSetup.ypos = ypos 
	end, name .. '.ui.Moved')
	
	Command.Event.Attach(EnKai.events[name .. '.ui'].Closed, function ()		
		nkCTSetup.visible = false
	end, name .. '.ui.Closed')
	
	Command.Event.Attach(EnKai.events[name .. '.ui'].Dragable, function (_, newFlag)		
		nkCTSetup.dragable = newFlag
	end, name .. '.ui.Dragable')
	
	local grid = EnKai.uiCreateFrame("nkGrid", name .. '.grid', ui:GetContent())
		
	grid:SetHeaderHeight(15)
	grid:SetHeaderFontSize(11)
	grid:SetFontSize(11)
	grid:SetPoint("TOPLEFT", ui:GetContent(), "TOPLEFT")
	grid:SetHeaderLabelColor(0.925, 0.894, 0.741, 1)
	grid:SetBorderColor(0, 0, 0, .6)
	grid:SetBodyColor(0.133, 0.133, 0.133, 1)
	
	local cols = {	{ align = 'left', width = 200, header = "Currency"  },
					{ align = 'center', width = 150, header = "Current"},	
					{ align = 'center', width = 50, header = "Max"}}
	
	grid:SetVisible(true)	
	grid:Layout (cols, 6)
	
	Command.Event.Attach(EnKai.events[name .. '.grid'].GridFinished, function ()
		local currencyInfo = Inspect.Currency.List()
		nkCurrency.CurrencyChange (nil, currencyInfo)
		ui:SetWidth(grid:GetWidth()+1)
	end, name .. '.grid.GridFinished')	
	
	Command.Event.Attach(EnKai.events[name .. '.grid'].LeftClick, function (_, rowNo)
		if nkCTSetup.titleCurrency == grid:GetKey(rowNo) then
			ui:SetTitleAlign ("right", -80)
			ui:SetTitle(string.format("nkCurrency V%s", addonInfo.toc.Version))
			nkCTSetup.titleCurrency = 'coin'
		else 				
			nkCTSetup.titleCurrency = grid:GetKey(rowNo)
			nkCurrency.CurrencyChange (nil, Inspect.Currency.List())
		end
	end, name .. '.grid.LeftClick')	
	
	local selection = {}
	for k, v in pairs (Inspect.Currency.Category.List()) do
		local details = Inspect.Currency.Category.Detail(k)
		if details ~= nil then
			table.insert(selection, { label = details.name, value = details.id})
		end
	end
	
	ui.categorySelect = EnKai.uiCreateFrame("nkCombobox", name .. '.categorySelect', ui:GetHeader())
	ui.categorySelect:SetPoint("CENTERLEFT", ui:GetHeader(), "CENTERLEFT", 45 , 0)
	ui.categorySelect:SetWidth(100)
	ui.categorySelect:SetHeight(16)
	ui.categorySelect:SetLabelWidth(0)	
	ui.categorySelect:SetSelection(selection)
	
	if nkCTSetup.selectedCurrency == nil then nkCTSetup.selectedCurrency = selection[1].value end
	
	ui.categorySelect:SetSelectedValue(nkCTSetup.selectedCurrency, false)
	ui.categorySelect:SetColor(0.925, 0.894, 0.741, 1)
	ui.categorySelect:SetLabelColor(0.925, 0.894, 0.741, 1)
	ui.categorySelect:SetColorSelected(0.925, 0.894, 0.741, 1)
	ui.categorySelect:SetLayer(9)
	
	Command.Event.Attach(EnKai.events[name .. '.categorySelect'].ComboChanged, function (_, newValue)
		nkCTSetup.selectedCurrency = newValue.value
		nkCurrency.CurrencyChange (nil, Inspect.Currency.List())
	end, name .. '.categorySelect.ComboChanged')
	
	function ui:PopulateGrid(gridValues)
		grid:SetRowPos(1, false)
		grid:SetCellValues(gridValues)
	end
	
	function ui:ShowGrid(flag)
		ui:ShowContent(flag)
	end

	return ui
	
end



